﻿#include "BinaryReader.h"

namespace edge2 {

// std::streamをソースにしてBinaryReaderを構築します。
BinaryReader::BinaryReader(std::ifstream& stream) {
	CheckEndian();

	// std::streamに対応したソースです。
	class StreamSource : public ISource {
	public:
		StreamSource(std::ifstream& stream)
			: stream_(stream) {
		}
		void Read(void* buffer, uint32_t size) override {
			stream_.read(reinterpret_cast<char*>(buffer), size);
			if (stream_.gcount() != size) {
				throw kException_Generic;
			}
		}
		int64_t position() override {
			return stream_.tellg();
		}
		void set_position(int64_t position) override {
			stream_.seekg(position, std::ios::beg);
			// メモ：シークしてもフラグ系はクリアされないので注意
		}
	private:
		std::ifstream& stream_;
	};

	source_ = new StreamSource(stream);
}

// std::vector<uint8_t>をソースにしてBinaryReaderを構築します。
BinaryReader::BinaryReader(const std::vector<uint8_t>& bytes) {
	CheckEndian();

	// std::vector<uint8_t>に対応したソースです。
	class BytesSource : public ISource {
	public:
		BytesSource(const std::vector<uint8_t>& bytes)
			: bytes_(bytes)
			, position_(0) {
		}
		void Read(void* buffer, uint32_t size) override {
			if (position_ < 0 || position_ + size > bytes_.size()) {
				throw kException_Generic;
			}
			memcpy(buffer, &bytes_[static_cast<unsigned int>(position_)], size);
			position_ += size;
		}
		int64_t position() override { return position_; }
		void set_position(int64_t position) override { 
			position_ = position;
		}
	private:
		const std::vector<uint8_t>& bytes_;
		int64_t position_;
	};

	source_ = new BytesSource(bytes);
}

BinaryReader::~BinaryReader(void) {
	if (source_) {
		delete source_;
		source_ = NULL;
	}
}

// このシステムのエンディアンを判定します。
void BinaryReader::CheckEndian() {
	uint16_t test = 0x0001;
	if (reinterpret_cast<uint8_t*>(&test)[0] == 1) {
		// リトルエンディアンなのでOK
	} else {
		// ビッグエンディアンは未対応
		throw kException_BigEndianNotSupport;
	}
}

}