﻿#pragma once

#include <cstdint>
#include <fstream>
#include <vector>
#include "Rgb.h"
#include "FrameLocationElement.h"

namespace edge2 {

// ifstreamからバイナリ形式を読み込みやすくするラッパークラスです。予期せぬ場合は例外をスローします。
class BinaryReader
{
public:
	enum Exceptions {
		// 汎用
		kException_Generic = 100,
		kException_BigEndianNotSupport,
	};

// 構築・消滅
public:
	BinaryReader(std::ifstream& stream);
	BinaryReader(const std::vector<uint8_t>& bytes);
	~BinaryReader(void);

// 読み込み操作
public:
	// 指定したサイズを読み込みます。足りない場合は例外をスローします。
	void Read(void* buffer, uint32_t size) { source_->Read(buffer, size); }
	// bool値を読み込みます。
	bool ReadBool() { return ReadUInt8() != 0; }
	// uint8値を読み込みます。
	uint8_t ReadUInt8() { uint8_t tmp; source_->Read(&tmp, sizeof(tmp)); return tmp; }
	// uint16値を読み込みます。
	uint16_t ReadUInt16() { uint16_t tmp; source_->Read(&tmp, sizeof(tmp)); return tmp; }
	// int32値を読み込みます。
	int32_t ReadInt32() { int32_t tmp; source_->Read(&tmp, sizeof(tmp)); return tmp; }
	// uint32値を読み込みます。
	uint32_t ReadUInt32() { uint32_t tmp; source_->Read(&tmp, sizeof(tmp)); return tmp; }
	// Rgb値を読み込みます。
	Rgb ReadRgb() {
		uint8_t b = ReadUInt8();
		uint8_t g = ReadUInt8();
		uint8_t r = ReadUInt8();
		return Rgb(r, g, b);
	}
	// FrameLocationElement値を読み込みます。
	FrameLocationElement ReadFrameLocationElement() {
		FrameLocationElement e;
		e.set_value(ReadInt32());
		e.set_relative(ReadBool());
		return e;
	}

// その他の操作
public:
	// 現在の読み取り位置を取得します。
	int64_t position() { return source_->position(); }
	// 読み取り位置をシークします。
	void set_position(int64_t position) { source_->set_position(position); }

private:
	// 読み込み用のアダプターインターフェース
	class ISource {
	public:
		ISource() {}
		virtual ~ISource() {}
		virtual void Read(void* buffer, uint32_t size) = 0;
		virtual int64_t position() = 0;
		virtual void set_position(int64_t position) = 0;
	};

private:
	// このシステムのエンディアンを判定します。
	void CheckEndian();

private:
	ISource* source_;
};

}