﻿/****************************************************************************
EDGE2 Data Loder Library
Copyright (C) 2015 Takabo Soft
****************************************************************************/

#include "zlib.h"
#include "Data.h"
#include "BinaryReader.h"

namespace edge2 {

Data::Data(void) 
	: bit_number_(0)
	, delay_unit_(Frame::kDelayUnit_60FPS)
	, page_fixed_palette_(false)
	, color_number_(0)
{
}

Data::~Data(void) {
	
}

// 指定ページ用のパレットを取得します。
Palette& Data::palette(Page& page) {
	if (page_fixed_palette_) {
		Palette* p = palettes_.Find(page.palette_link());
		if (p) {
			return *p;
		}
	}
	return palettes_.current();
}

// ファイルからEDGE2ファイルを読み込みます。エラー値が戻ります。
Data::ReadResults Data::Read(const char *path) {
	try {
		std::ifstream is(path, std::ios_base::in | std::ios_base::binary);
		if (!is) {
			return kReadResult_FileCannotOpen;
		}

		BinaryReader reader(is);

		// 識別子を確認します。
		char identifier[5];
		reader.Read(identifier, 5);
		if (memcmp(identifier, "EDGE2", 5)) {
			return kReadResult_NotEdge2File;
		}

		// Edge2ファイルフォーマットのローカルバージョンを取得します。
		if (reader.ReadUInt16() != 0) {
			// 2015/07/05現在 0 のみ。
			return kReadResult_LocalVersionNotSupport;
		}

		// 圧縮モードを確認します。
		if (reader.ReadUInt8() != kCompressionMode_ZLIB) {
			// 2015/07/05現在 ZLIB のみ。
			return kReadResult_CompressModeNotSupport;
		}

		// 圧縮データをメモリへ展開します（負荷になるようならばテンポラリファイルへ書き出す方法も有り）。
		std::vector<uint8_t> bytes;
		if (!InflateByZLIB(is, bytes, reader.ReadUInt32())) {
			return kReadResult_ZLIBInflateFailed;
		}
		is.close();

		// 展開したデータのチャンクを読み込みます。
		BinaryReader bytes_reader(bytes);
		ChunkSerializer::ReadChunks(*this, bytes_reader, static_cast<uint32_t>(bytes.size()));

		return kReadResult_Success;

	} catch (BinaryReader::Exceptions /*br_exp*/) {
		return kReadResult_BinaryReaderFailed;
	} catch (ReadResults result) {
		return result;
	} catch (...) {
		return kReadResult_Failed;
	}
	return kReadResult_Failed;
}

// ZLIBを使って圧縮データを展開します。	uncompressではなく、少しずつ読み込んで展開します。
bool Data::InflateByZLIB(std::ifstream& is, std::vector<uint8_t>& output_bytes, uint32_t output_size) {
	
	z_stream z = {0};
	if (inflateInit(&z) != Z_OK) {
		return false;
	}

	output_bytes.resize(output_size);
	z.next_out = &output_bytes[0];
	z.avail_out = output_size;

	const int comp_buffer_size = 1024 * 10;
	std::vector<uint8_t> comp_buffer(comp_buffer_size);
	
	int status = Z_OK;

	while (status != Z_STREAM_END) {
		if (z.avail_in == 0 && !is.eof()) {
			z.next_in = &comp_buffer[0];
			is.read(reinterpret_cast<char*>(&comp_buffer[0]), comp_buffer_size);
			z.avail_in = static_cast<uInt>(is.gcount());
		}
		status = inflate(&z, Z_NO_FLUSH);
		if (status == Z_STREAM_END) {
			break;
		}
		if (status != Z_OK) {
			inflateEnd(&z);
			return false;
		}
	}

	if (z.avail_out != 0) {
		inflateEnd(&z);
		return false;
	}

	inflateEnd(&z);
	return true;
}

// チャンクデータの読み込むために呼ばれます。
void Data::ReadChunk(BinaryReader& reader, uint16_t chunk_id, uint32_t length) {
	switch (chunk_id) {
	case kChunkID_BitNumber:
		bit_number_ = reader.ReadUInt8();
		break;
	case kChunkID_ColorPrecision:
		// ローダーは未対応
		break;
	case kChunkID_ColorAlpha:
		// 未使用
		break;
	case kChunkID_DelayUnit:
		delay_unit_ = static_cast<Frame::DelayUnits>(reader.ReadUInt8());
		break;
	case kChunkID_SearchUserColorTarget:
		// ローダーは未対応
		break;
	case kChunkID_SaveOption:
		// ローダーは未対応
		break;
	case kChunkID_ExportOption:
		// ローダーは未対応
		break;
	case kChunkID_PaletteSaveOption:
		// ローダーは未対応
		break;
	case kChunkID_PageCount:
		// 未使用
		break;
	case kChunkID_PageNextID:
		// ローダーは未対応
		break;
	case kChunkID_PageFixedPalette:
		page_fixed_palette_ = reader.ReadBool();
		break;
	case kChunkID_Page:
		{
			Page* page = new Page();
			page->set_bit_number(bit_number_);
			ChunkSerializer::ReadChunks(*page, reader, length);
			pages_.PushBack(page);
		}
		break;
	case kChunkID_PageCurrentIndex:
		pages_.set_current_index(reader.ReadInt32());
		break;
	case kChunkID_PaletteCount:
		// 未使用
		break;
	case kChunkID_PaletteColorCount:
		color_number_ = reader.ReadUInt16();
		break;
	case kChunkID_PaletteNextID:
		// ローダーは未対応
		break;
	case kChunkID_Palette:
		{
			Palette* palette = new Palette();
			palettes_.PushBack(palette);
			if (!palette->Create(color_number_)) {
				throw kReadResult_Failed;
			}
			ChunkSerializer::ReadChunks(*palette, reader, length);
		}
		break;
	case kChunkID_PaletteCurrentIndex:
		palettes_.set_current_index(reader.ReadInt32());
		break;
	case kChunkID_PaletteDrawColor:
		// 互換性のため
		draw_color_.index = static_cast<uint8_t>(reader.ReadUInt32());
		break;
	case kChunkID_PaletteBackColor:
		// 互換性のため
		back_color_.index = static_cast<uint8_t>(reader.ReadUInt32());
		break;
	case kChunkID_PaletteDrawColor2:
		ChunkSerializer::ReadChunks(draw_color_, reader, length);
		break;
	case kChunkID_PaletteBackColor2:
		ChunkSerializer::ReadChunks(back_color_, reader, length);
		break;
	case kChunkID_TipUnit:
		// ローダーは未対応
		break;
	case kChunkID_FrameCoordinateSystem:
		ChunkSerializer::ReadChunks(frame_coordinate_system_, reader, length);
		break;
	case kChunkID_IndexRangeEnable:
		// ローダーは未対応
		break;
	case kChunkID_IndexRangeCount:
		// ローダーは未対応
		break;
	case kChunkID_IndexRangeNextID:
		// ローダーは未対応
		break;
	case kChunkID_IndexRange:
		// ローダーは未対応
		break;
	case kChunkID_IndexRangeCurrent:
		// ローダーは未対応
		break;
	}
}

}