﻿#pragma once

#include <cstdint>
#include <vector>
#include "Uncopyable.h"
#include "ChunkSerializer.h"
#include "Frame.h"
#include "Items.h"
#include "Page.h"
#include "Palette.h"
#include "DrawBackColor.h"
#include "FrameCoordinateSystem.h"

namespace edge2 {

// EDGE2のデータクラスです。
class Data : public Uncopyable, public IChunkSerializable {
public:
	enum ReadResults {
		// 成功です。
		kReadResult_Success = 0,

		// 以下、エラーです。

		// 何かしら失敗したことを意味します。
		kReadResult_Failed,
		// バイナリ読み込み中に何らかのエラーが発生しました。
		kReadResult_BinaryReaderFailed,
		// ファイルのオープンに失敗しました。
		kReadResult_FileCannotOpen,
		// Edge2ファイルではありません。
		kReadResult_NotEdge2File,
		// Edge2のローカルバージョンに対応していません。
		kReadResult_LocalVersionNotSupport,
		// 指定された圧縮モードには対応していません。
		kReadResult_CompressModeNotSupport,
		// ZLIBによる展開処理に失敗しました。
		kReadResult_ZLIBInflateFailed,
	};

	// 圧縮モードを確認します。
	enum CompressionModes {
		kCompressionMode_None = 0,
		kCompressionMode_ZLIB,
	};

	// チャンクID（固定値）
	enum ChunkIDs {
		kChunkID_BitNumber = 1000,
		kChunkID_ColorPrecision,
		kChunkID_ColorAlpha,
		kChunkID_DelayUnit,
		kChunkID_SearchUserColorTarget,
		kChunkID_SaveOption,
		kChunkID_ExportOption,
		kChunkID_PaletteSaveOption,
		
		kChunkID_PageCount = 2000,
		kChunkID_PageNextID,
		kChunkID_PageFixedPalette,
		kChunkID_Page,
		kChunkID_PageCurrentIndex,

		kChunkID_PaletteCount = 3000,
		kChunkID_PaletteColorCount,
		kChunkID_PaletteNextID,
		kChunkID_Palette,
		kChunkID_PaletteCurrentIndex,
		kChunkID_PaletteDrawColor,	// フルカラー対応の際に未使用となりました。
		kChunkID_PaletteBackColor,	// フルカラー対応の際に未使用となりました。
		kChunkID_PaletteDrawColor2,
		kChunkID_PaletteBackColor2,

		kChunkID_TipUnit = 4000,
		kChunkID_FrameCoordinateSystem,

		kChunkID_IndexRangeEnable = 5000,
		kChunkID_IndexRangeCount,
		kChunkID_IndexRangeNextID,
		kChunkID_IndexRange,
		kChunkID_IndexRangeCurrent,
	};

// 構築・消滅
public:
	Data(void);
	~Data(void);

// アクセサ
public:
	// 背景色の参照を取得します。
	DrawBackColor& back_color() { return back_color_; }
	// ページコンテナを取得します。
	Items<Page>& pages() { return pages_; }
	// パレットコンテナを取得します。
	Items<Palette>& palettes() { return palettes_; }
	// 指定ページ用のパレットを取得します。
	Palette& palette(Page& page);
	// ページ固定パレットかどうかのフラグを取得します。
	bool page_fixed_palette() { return page_fixed_palette_; }
	// フレーム座標系を取得します。
	FrameCoordinateSystem& frame_coordinate_system() { return frame_coordinate_system_; }

// 操作
public:
	// ファイルからEDGE2ファイルを読み込みます。エラー値が戻ります。
	ReadResults Read(const char *path);

// ツール
private:
	// ZLIBを使って圧縮データを展開します。	
	bool InflateByZLIB(std::ifstream& is, std::vector<uint8_t>& output_bytes, uint32_t output_size);

// IChunkSerializable
public:
	// チャンクデータの読み込むために呼ばれます。
	void ReadChunk(BinaryReader& reader, uint16_t chunk_id, uint32_t length);

// メンバ変数
private:
	// 色のビット数(1, 4, 8, 15, 16, 24) 
	uint8_t bit_number_;
	// ディレイ単位
	Frame::DelayUnits delay_unit_;
	// ページ固定パレットかどうかのフラグ
	bool page_fixed_palette_;
	// ページコンテナ
	Items<Page> pages_;
	// 色登録数(1～256)
	int color_number_;
	// パレットコンテナ
	Items<Palette> palettes_;
	// 描画色
	DrawBackColor draw_color_;
	// 背景色
	DrawBackColor back_color_;
	// フレーム座標系
	FrameCoordinateSystem frame_coordinate_system_;
};

}