﻿#include "Page.h"
#include "BinaryReader.h"
#include "Data.h"

namespace edge2 {

Page::Page(void)
	: bit_number_(0)
	, page_id_(0)
	, width_(0)
	, height_(0)
	, layer_visible_mode_(kLayerVisibleMode_None)
{
}

Page::~Page(void) {
}

// レイヤー可視モード込みで指定したレイヤーが可視かどうかを判定します。
bool Page::layer_visible(int layer_index) {

	switch (layer_visible_mode_) {

	// レイヤー毎に可視を設定する
	case kLayerVisibleMode_None:
		if (layers_[layer_index].visible()) {
			// グループ化されている場合、親の影響を受ける
			if (layers_[layer_index].grouped()) {
				int start_index, end_index;
				layers_.group_range(layer_index, start_index, end_index);
				return layers_[start_index].visible();
			} else {
				return true;
			}
		}
		break;

	// カレントレイヤーのみ
	case kLayerVisibleMode_Current:
		return layer_index == layers_.current_index();
	
	// カレントレイヤーと一つ上のレイヤーのみ
	case kLayerVisibleMode_CurrentAndUp:
		return layer_index == layers_.current_index() || layer_index == layers_.current_index() - 1;
		
	// カレントレイヤーと一つ下のレイヤーのみ
	case kLayerVisibleMode_CurrentAndDown:
		return layer_index == layers_.current_index() || layer_index == layers_.current_index() + 1;

	// カレントレイヤー以上のみ
	case kLayerVisibleMode_OverCurrent:
		return layer_index <= layers_.current_index();
	
	// カレントレイヤー以下のみ
	case kLayerVisibleMode_UnderCurrent:
		return layer_index >= layers_.current_index();
	
	// カレントレイヤーが所属するグループのみ
	case kLayerVisibleMode_CurrentGroup:
		{
			int start_index, end_index;
			layers_.group_range(layers_.current_index(), start_index, end_index);
			return layer_index >= start_index && layer_index < end_index;
		}
		break;
	}
	return false;
}

// レイヤーが重なった状態の画像を生成します。
bool Page::CreateMergedImage(Image& image, const DrawBackColor& backColor) {
	if (!image.Create(width_, height_, bit_number_)) {
		return false;
	}
	image.Clear(backColor);
	for (int layer_index = layers_.size() - 1; layer_index >= 0; layer_index--) {
		if (layer_visible(layer_index)) {
			layers_[layer_index].image().Draw(image, backColor);
		}
	}
	return true;
}

// チャンクデータの読み込むために呼ばれます。
void Page::ReadChunk(BinaryReader& reader, uint16_t chunk_id, uint32_t length) {
	switch (chunk_id) {
	case kChunkID_Name:
		// ローダーは未対応
		break;
	case kChunkID_ID:
		page_id_ = reader.ReadInt32();
		break;
	case kChunkID_Grouped:
		set_grouped(reader.ReadBool());
		break;
	case kChunkID_GroupExpansion:
		// ローダーは未対応
		break;
	case kChunkID_Selected:
		// ローダーは未対応
		break;
	case kChunkID_Width:
		width_ = reader.ReadUInt16();
		break;
	case kChunkID_Height:
		height_ = reader.ReadUInt16();
		break;
	case kChunkID_PaletteLinkID:
		palette_link_.set_item_id(reader.ReadInt32());
		break;
	case kChunkID_PaletteLinkIndex:
		palette_link_.set_index_cache(reader.ReadInt32());
		break;
	case kChunkID_ItemBgColor:
		// ローダーは未対応
		// ※ページウィンドウ上での行の色
		break;
	case kChunkID_LayerVisibleMode:
		layer_visible_mode_ = static_cast<LayerVisibleModes>(reader.ReadUInt8());
		break;
	case kChunkID_LayerNextID:
		// ローダーは未対応
		break;
	case kChunkID_LayerCount:
		// 未使用
		break;
	case kChunkID_Layer:
		{
			Layer* layer = new Layer();
			layers_.PushBack(layer);
			if (!layer->image().Create(width_, height_, bit_number_)) {
				throw Data::kReadResult_Failed;
			}
			ChunkSerializer::ReadChunks(*layer, reader, length);
		}
		break;
	case kChunkID_LayerCurrentIndex:
		layers_.set_current_index(reader.ReadInt32());
		break;
	case kChunkID_FrameDelay:
		frame_.set_delay(reader.ReadInt32());
		break;
	case kChunkID_FrameTransparent:
		frame_.set_transparent(reader.ReadBool());
		break;
	case kChunkID_FrameDisposalMethod:
		frame_.set_disposal_method(static_cast<Frame::DisposalMethods>(reader.ReadUInt8()));
		break;
	case kChunkID_FrameX:
		frame_.location().set_x(reader.ReadFrameLocationElement());
		break;
	case kChunkID_FrameY:
		frame_.location().set_y(reader.ReadFrameLocationElement());
		break;
	case kChunkID_FrameUse:
		frame_.set_use(reader.ReadBool());
		break;
	case kChunkID_TipSize:
		// ローダーは未対応
		break;
	case kChunkID_TipPalette:
		// ローダーは未対応
		break;
	}
}

}

